local L = AceLibrary("AceLocale-2.2"):new("FuBar_WindFuryFu")
local T = AceLibrary("Tablet-2.0")
local BS = AceLibrary("Babble-Spell-2.2")
local BC = AceLibrary("Babble-Class-2.2")
local G = AceLibrary("Gratuity-2.0")
local D = AceLibrary("Deformat-2.0")

WindFuryFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0")
WindFuryFu.version = "2.0"
WindFuryFu.revision  = tonumber(string.sub("$Rev: 27241 $", 12, -3))
WindFuryFu.date = "$LastChangedDate: 2007-02-05 06:19:20 -0500 (Mon, 05 Feb 2007) $"
WindFuryFu.hasIcon = true
WindFuryFu.canHideText = true
WindFuryFu.independentProfile = true
--WindFuryFu.clickableTooltip = true
WindFuryFu:RegisterDB("WindFuryFuDB")
WindFuryFu:RegisterDefaults('profile', {
	debugEnabled = false,
	showLabel = true,
	displayLast = true,
	displayBest = true,
	displayChat = true,
	tooltipVictims = true,
	tooltipLastFive = true,
	tooltipShowHint = true,
	useSCT = true,
	fontSize = 26,
	fontFace = "Default",
	fontColor = {r=256,g=256,b=0},
	countFirstHit = true,
})
WindFuryFu:RegisterDefaults('char', {
	WF = {},
	WFS = {}
})
WindFuryFu.options = {
	handler = WindFuryFu,
	type = 'group',
	args = {
		display = {
			type = 'group',
			name = L["Display Options"],
			desc = L["Set the Display Properties"],
			args = {
				debugEnabled = {
					type = 'toggle',
					name = L["Show Debug Messages"],
					desc = L["Toggle showing of debug messages in chat frame"],
					get = function() return WindFuryFu.db.profile.debugEnabled end,
					set = function(t) WindFuryFu.db.profile.debugEnabled = t end,
				},
				label = {
					type = 'toggle',
					name = L["Show Label Text"],
					desc = L["Toggle text labels on FuBar"],
					get = function() return WindFuryFu.db.profile.showLabel end,
					set = function(t)
						WindFuryFu.db.profile.showLabel = t
						WindFuryFu:UpdateText()
					end,
				},
				last = {
					type = 'toggle',
					name = L["Show Last WF Hit"],
					desc = L["Toggle showing your last Windfury hit"],
					get = function() return WindFuryFu.db.profile.displayLast end,
					set = function(t)
						WindFuryFu.db.profile.displayLast = t
						WindFuryFu:UpdateText()
					end,
				},
				best = {
					type = 'toggle',
					name = L["Show Best WF Hit"],
					desc = L["Toggle showing your best Windfury hit"],
					get = function() return WindFuryFu.db.profile.displayBest end,
					set = function(t)
						WindFuryFu.db.profile.displayBest = t
						WindFuryFu:UpdateText()
					end,
				},
				chat = {
					type = 'toggle',
					name = L["Display Total Damage in Chat Frame"],
					desc = L["Toggle displaying total damage chat frame"],
					get = function() return WindFuryFu.db.profile.displayChat end,
					set = function(t) WindFuryFu.db.profile.displayChat = t end,
				},	
			},
		},
		tooltip = {
			type = 'group',
			name = L["Tooltip Options"],
			desc = L["Set the Tooltip Properties"],
			args = {
				lastfive = {
					type = 'toggle',
					name = L["Show Five Last WF Hits"],
					desc = L["Toggle showing the last five Windfury hits in the tooltip"],
					get = function() return WindFuryFu.db.profile.tooltipLastFive end,
					set = function(t)
						WindFuryFu.db.profile.tooltipLastFive = t
						WindFuryFu:UpdateTooltip()
					end,
				},
				victims = {
					type = 'toggle',
					name = L["Show Victims"],
					desc = L["Toggle showing victims for each attack in the tooltip"],
					get = function() return WindFuryFu.db.profile.tooltipVictims end,
					set = function(t)
						WindFuryFu.db.profile.tooltipVictims = t
						WindFuryFu:UpdateTooltip()
					end,
				},
				hint = {
					type = 'toggle',
					name = L["Show Hint"],
					desc = L["Toggle showing the hint in the tooltip"],
					get = function() return WindFuryFu.db.profile.tooltipShowHint end,
					set = function(t)
						WindFuryFu.db.profile.tooltipShowHint = t
						WindFuryFu:UpdateTooltip()
					end,
				},

			},
		},
		stats = {
			type = 'group',
			name = L["Statistic Options"],
			desc = L["Configure how statistics should be monitored and saved"],
			args = {
				countfirsthit = {
					type = 'toggle',
					name = L["Count First Hit"],
					desc = L["Count the hit that triggered the Windfury attack and add it to total damage"],
					get = function() return WindFuryFu.db.profile.countFirstHit end,
					set = function(t) WindFuryFu.db.profile.countFirstHit = t end,
				},
				reset = {
					type = 'group',
					name = L["Reset Statistics"],
					desc = L["Reset your Windfury statistics"],
					args = {
						session = {
							type = 'execute',
							name = L["Session"],
							desc = L["Reset your Windfury statistics for this session"],
							func = function()
								WindFuryFu:ResetStats("SESSION")
							end
						},
						lifetime = {
							type = 'execute',
							name = L["Lifetime"],
							desc = L["Reset your lifetime Windfury statistics"],
							func = function()
								WindFuryFu:ResetStats("LIFETIME")
							end
						},
					},
				},
			},
		},
		scrollingtext = {
			type = 'group',
			name = L["SCT/MSBT Options"],
			desc = L["Set the Display Properties for SCT/MSBT"],
			args = {
				toggle = {
					type = 'toggle',
					name = L["Enable Notification"],
					desc = L["Toggle showing total windfury damage notification in SCT or MSBT"],
					get = function() return WindFuryFu.db.profile.useSCT end,
					set = function(t) WindFuryFu.db.profile.useSCT = t end,
				},
				fontsize = {
					type = 'range',
					name = L["Font Size"],
					desc = L["Set the font size to use when displaying messages in SCT/MSBT"],
					step = 1,
					min = 8,
					max = 32,
					get = function() return WindFuryFu.db.profile.fontSize end,
					set = function(v) WindFuryFu.db.profile.fontSize = v end,
				},
				fontface = {
					type = 'text',
					name = L["Font Face"],
					desc = L["Set the font face to use when displaying messages in SCT/MSBT"],
					usage = "Default",
					get = function() return WindFuryFu.db.profile.fontFace end,
					set = function(t) WindFuryFu.db.profile.fontFace = t end,
				},
				fontcolor = {
					type = 'color',
					name = L["Font Color"],
					desc = L["Set the font color to use when displaying messages in SCT/MSBT"],
					get = function() return WindFuryFu.db.profile.fontColor.r,WindFuryFu.db.profile.fontColor.g, WindFuryFu.db.profile.fontColor.b end,
					set = function(r,g,b)
						WindFuryFu.db.profile.fontColor.r = r
						WindFuryFu.db.profile.fontColor.g = g
						WindFuryFu.db.profile.fontColor.b = b
					end,
					hasAlpha = false,
				},
			},
		},
	},
}
WindFuryFu:RegisterChatCommand(L["AceConsole-options"], WindFuryFu.options)
WindFuryFu.OnMenuRequest = WindFuryFu.options

function WindFuryFu:OnEnable()
	self:RegisterEvent("UNIT_INVENTORY_CHANGED", "InventoryChanged")
	self:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE", "SpellSelfDamage")
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS", "CombatSelfHit")
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH", "CombatHostileDeath")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "TargetChanged")
	self:RegisterEvent("PLAYER_ENTER_COMBAT", "EnterCombat")

	self.ACTIVE_WEAPON= "none"
	self.MAINHAND = ""
	self.OFFHAND = ""
	self.MAINHAND_EN = false
	self.OFFHAND_EN = false
	self.ACTIVE_WEAPON= ""
	self.STATUS = false
	self.DUAL_HIT = false
	self.DUAL_WIELD = false
	self.LASTHIT = 0
	self.LASTHIT_CRIT = false
	self.SHIT = 0
	self.CRIT = 0
	self.SWINGS_LEFT = 0
	self.THIS_TOTAL = 0
	
	self.WF_VICTIM = {
		name = "",
		class = "",
		level = 0,
		dead = false
	}

	self:WeaponCheck()
	self:ResetStats("SESSION")
end

function WindFuryFu:OnClick()
	if (IsShiftKeyDown() and ChatFrameEditBox:IsVisible()) then
		-- Needs to be localized
		message = self.MAINHAND.." Stats - Best WF: "..self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST"].." - Average WF dmg: "..self.WF_ALL_AVERAGE.." - Best hit during WF: "..self.db.char.WFS[self.MAINHAND]["WF_ALL_SHIT"]
		ChatFrameEditBox:Insert(message)
	end
end

function WindFuryFu:OnDataUpdate()
	if (self.db.char.WF[self.MAINHAND]["TOTAL"] == 0) then
		self.WF_AVERAGE = "0"
	else
		self.WF_AVERAGE = ceil(self.db.char.WF[self.MAINHAND]["TOTAL"] / self.db.char.WF[self.MAINHAND]["PROCS"])
	end

	if (self.db.char.WFS[self.MAINHAND]["WF_ALL_TOTAL"] == 0) then
		self.WF_ALL_AVERAGE = "0"
	else
		self.WF_ALL_AVERAGE = ceil(self.db.char.WFS[self.MAINHAND]["WF_ALL_TOTAL"] / self.db.char.WFS[self.MAINHAND]["WF_ALL_PROCS"])
	end

	if (self.db.char.WF[self.MAINHAND]["CRITS"] == 0) then
		self.WF_OUT_CRITS = "0"
	else
		self.WF_OUT_CRITS = ceil((self.db.char.WF[self.MAINHAND]["CRITS"] / (self.db.char.WF[self.MAINHAND]["PROCS"]*3))*100)
	end

	if (self.db.char.WF[self.MAINHAND]["PROCS"] == 0) then
		self.WF_OUT_PROCS = "0"
	else
		self.WF_OUT_PROCS = ceil((self.db.char.WF[self.MAINHAND]["PROCS"] / self.db.char.WF[self.MAINHAND]["HITS"])*100)
	end

	if (self.db.char.WFS[self.MAINHAND]["WF_ALL_PROCS"] == 0) then
		self.WF_ALL_OUT_PROCS = "0"
	else
		self.WF_ALL_OUT_PROCS = ceil((self.db.char.WFS[self.MAINHAND]["WF_ALL_PROCS"] / self.db.char.WFS[self.MAINHAND]["WF_ALL_HITS"])*100)
	end

	if (self.db.char.WFS[self.MAINHAND]["WF_ALL_CRITS"] == 0) then
		self.WF_ALL_OUT_CRITS = "0"
	else
		self.WF_ALL_OUT_CRITS = ceil((self.db.char.WFS[self.MAINHAND]["WF_ALL_CRITS"] / (self.db.char.WFS[self.MAINHAND]["WF_ALL_PROCS"]*3))*100)
	end
end

function WindFuryFu:OnTextUpdate()
	local str = ""

	if (self.db.profile.showLabel) then
		if (self.db.profile.displayLast) then
			str = str .. L["Last"] ..": |c00FFFFFF" .. self.db.char.WF[self.MAINHAND]["LAST"][1] .. "|r"
		end
		if (self.db.profile.displayBest) then
			if (strlen(str) > 4) then
				str = str .. "  "
			end
			if (self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST"] < self.db.char.WF[self.MAINHAND]["BEST"]) then
				str = str .. L["Best"] ..": |c00FFFFFF" .. self.db.char.WF[self.MAINHAND]["BEST"] .. "|r"
			else
				str = str .. L["Best"] ..": |c00FFFFFF" .. self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST"] .. "|r"
			end
		end
		if (self.STATUS) then
			str = str .. " " .. L["Status"] .. ": |cff20ff20ON|r"
		else
			str = str .. " " .. L["Status"] .. ": |cffff2020OFF|r"
		end
	else
		if (self.db.profile.displayLast) then
			str = str .. "|c00FFFFFF" .. self.db.char.WF[self.MAINHAND]["LAST"][1] .. "|r/"
		end
		if (self.db.profile.displayBest) then
			if (self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST"] < self.db.char.WF[self.MAINHAND]["BEST"]) then
				str = str .. "|c00FFFFFF" .. self.db.char.WF[self.MAINHAND]["BEST"] .. "|r/"
			else
				str = str .. "|c00FFFFFF" .. self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST"] .. "|r/"
			end
		end
		if (self.STATUS) then
			str = str .. "|cff20ff20ON|r"
		else
			str = str .. "|cffff2020OFF|r"
		end
	end
	self:SetText(str)
end

function WindFuryFu:OnTooltipUpdate()
	local mainHandLink = GetInventoryItemLink("player", GetInventorySlotInfo("MainHandSlot"))
	local offHandLink = GetInventoryItemLink("player", GetInventorySlotInfo("SecondaryHandSlot"))
	local cat = nil

	if (self.DUAL_WIELD) then
		cat = T:AddCategory(
			'columns', 1,
			'text', L["Weapons (Dual-wield)"]
		)
	else
		cat = T:AddCategory(
			'columns', 1,
			'text', L["Weapon"]
		)
	end
	
	if (mainHandLink == nil) and (offHandLink == nil) then
		cat:AddLine(
			'text', L["No Weapon Equipped"]
		)
	else
		if ((self.MAINHAND ~= "") and (mainHandLink ~= nil)) then
			cat:AddLine(
				'text', mainHandLink
			)
		end
		if ((self.OFFHAND ~= "") and (offHandLink ~= nil))  then
			cat:AddLine(
				'text', offHandLink
			)
		end
	end
	
	if (self.db.profile.tooltipVictims) then
		cat = T:AddCategory(
			'columns', 3,
			'text', L["This Session"],
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 0,
			'child_text2R', 1,
			'child_text2G', 1,
			'child_text2B', 1,
			'child_text3R', 1,
			'child_text3G', 1,
			'child_text3B', 1
		)
	
		local victim = self.db.char.WF[self.MAINHAND]["LAST_VICTIM"][1]
		local r3, g3, b3 = BC:GetColor(victim.class)
		cat:AddLine(
			'text', L["Last"]..":",
			'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][1] .. ") "..self.db.char.WF[self.MAINHAND]["LAST"][1],
			'text3', victim.name ..  " [|cffffffff" .. victim.level .. "|r]",
			'text3R', r3,
			'text3G', g3,
			'text4B', b3
		)
		
		if (self.db.profile.tooltipLastFive) then
			victim = self.db.char.WF[self.MAINHAND]["LAST_VICTIM"][2]
			r3, g3, b3 = BC:GetColor(victim.class)
			cat:AddLine(
				'text', L["2nd Last"]..":",
				'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][2]..") "..self.db.char.WF[self.MAINHAND]["LAST"][2],
				'text3', victim.name ..  " [|cffffffff" .. victim.level .. "|r]",
				'text3R', r3,
				'text3G', g3,
				'text4B', b3
			)
			victim = self.db.char.WF[self.MAINHAND]["LAST_VICTIM"][3]
			r3, g3, b3 = BC:GetColor(victim.class)
			cat:AddLine(
				'text', L["3rd Last"]..":",
				'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][3]..") "..self.db.char.WF[self.MAINHAND]["LAST"][3],
				'text3', victim.name ..  " [|cffffffff" .. victim.level .. "|r]",
				'text3R', r3,
				'text3G', g3,
				'text4B', b3
			)
			local victim = self.db.char.WF[self.MAINHAND]["LAST_VICTIM"][4]
			r3, g3, b3 = BC:GetColor(victim.class)
			cat:AddLine(
				'text', L["4th Last"]..":",
				'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][4]..") "..self.db.char.WF[self.MAINHAND]["LAST"][4],
				'text3', victim.name ..  " [|cffffffff" .. victim.level .. "|r]",
				'text3R', r3,
				'text3G', g3,
				'text4B', b3
			)
			victim = self.db.char.WF[self.MAINHAND]["LAST_VICTIM"][5]
			r3, g3, b3 = BC:GetColor(victim.class)
			cat:AddLine(
				'text', L["5th Last"]..":",
				'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][5]..") "..self.db.char.WF[self.MAINHAND]["LAST"][5],
				'text3', victim.name ..  " [|cffffffff" .. victim.level .. "|r]",
				'text3R', r3,
				'text3G', g3,
				'text4B', b3
			)
		end

		victim = self.db.char.WF[self.MAINHAND]["SHIT_VICTIM"]
		r3, g3, b3 = BC:GetColor(victim.class)
		cat:AddLine(
			'text', L["Best Single Hit"]..":",
			'text2', self.db.char.WF[self.MAINHAND]["SHIT"],
			'text3', victim.name ..  " [|cffffffff" .. victim.level .. "|r]",
			'text3R', r3,
			'text3G', g3,
			'text4B', b3
		)
		victim = self.db.char.WF[self.MAINHAND]["BEST_VICTIM"]
		r3, g3, b3 = BC:GetColor(victim.class)
		cat:AddLine(
			'text', L["Best"]..":",
			'text2', self.db.char.WF[self.MAINHAND]["BEST"],
			'text3', victim.name ..  " [|cffffffff" .. victim.level .. "|r]",
			'text3R', r3,
			'text3G', g3,
			'text4B', b3
		)
	else
		cat = T:AddCategory(
			'columns', 2,
			'text', L["This Session"],
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 0,
			'child_text2R', 1,
			'child_text2G', 1,
			'child_text2B', 1
		)

		cat:AddLine(
			'text', L["Last"]..":",
			'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][1] .. ") "..self.db.char.WF[self.MAINHAND]["LAST"][1]
		)
		
		if (self.db.profile.tooltipLastFive) then
			cat:AddLine(
				'text', L["2nd Last"]..":",
				'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][2]..") "..self.db.char.WF[self.MAINHAND]["LAST"][2]
			)
			cat:AddLine(
				'text', L["3rd Last"]..":",
				'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][3]..") "..self.db.char.WF[self.MAINHAND]["LAST"][3]
			)
			cat:AddLine(
				'text', L["4th Last"]..":",
				'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][4]..") "..self.db.char.WF[self.MAINHAND]["LAST"][4]
			)
			cat:AddLine(
				'text', L["5th Last"]..":",
				'text2', "("..self.db.char.WF[self.MAINHAND]["CRIT"][5]..") "..self.db.char.WF[self.MAINHAND]["LAST"][5]
			)
		end
		cat:AddLine(
			'text', L["Best Single Hit"]..":",
			'text2', self.db.char.WF[self.MAINHAND]["SHIT"]
		)
		cat:AddLine(
			'text', L["Best"]..":",
			'text2', self.db.char.WF[self.MAINHAND]["BEST"]
		)
	end
	cat:AddLine(
		'text', L["Procs"]..":",
		'text2', "("..self.db.char.WF[self.MAINHAND]["PROCS"].."/"..self.db.char.WF[self.MAINHAND]["HITS"]..") "..self.WF_OUT_PROCS.."%"
	)
	cat:AddLine(
		'text', L["Crits"]..":",
		'text2', "("..self.db.char.WF[self.MAINHAND]["CRITS"].."/"..(self.db.char.WF[self.MAINHAND]["PROCS"]*3)..") "..self.WF_OUT_CRITS.."%"
	)
	cat:AddLine(
		'text', L["Average"]..":",
		'text2', self.WF_AVERAGE
	)
	cat:AddLine(
		'text', L["Total WF Damage"]..":",
		'text2', self.db.char.WF[self.MAINHAND]["TOTAL"]
	)

	if (self.db.profile.tooltipVictims) then
		cat = T:AddCategory(
			'columns', 3,
			'text', L["Lifetime"],
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 0,
			'child_text2R', 1,
			'child_text2G', 1,
			'child_text2B', 1,
			'child_text3R', 1,
			'child_text3G', 1,
			'child_text3B', 1
		)
	 
		victim = self.db.char.WFS[self.MAINHAND]["WF_ALL_SHIT_VICTIM"]
		r3, g3, b3 = BC:GetColor(victim.class)
		cat:AddLine(
			'text', L["Best Single Hit"]..":",
			'text2', self.db.char.WFS[self.MAINHAND]["WF_ALL_SHIT"],
			'text3', victim.name ..  " [|cffffffff" .. victim.level .. "|r]",
			'text3R', r3,
			'text3G', g3,
			'text4B', b3
		)
		
		victim = self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST_VICTIM"]
		r3, g3, b3 = BC:GetColor(victim.class)
		cat:AddLine(
			'text', L["Best"]..":",
			'text2', self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST"],
			'text3', victim.name ..  " [|cffffffff" .. victim.level .. "|r]",
			'text3R', r3,
			'text3G', g3,
			'text4B', b3
		)
	else
		cat = T:AddCategory(
			'columns', 2,
			'text', L["Lifetime"],
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 0,
			'child_text2R', 1,
			'child_text2G', 1,
			'child_text2B', 1
		)
	 
		cat:AddLine(
			'text', L["Best Single Hit"]..":",
			'text2', self.db.char.WFS[self.MAINHAND]["WF_ALL_SHIT"]
		)
		cat:AddLine(
			'text', L["Best"]..":",
			'text2', self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST"]
		)
	end
	cat:AddLine(
		'text', L["Procs"]..":",
		'text2', "("..self.db.char.WFS[self.MAINHAND]["WF_ALL_PROCS"].."/"..self.db.char.WFS[self.MAINHAND]["WF_ALL_HITS"]..") "..self.WF_ALL_OUT_PROCS.."%"
	)
	cat:AddLine(
		'text', L["Crits"]..":",
		'text2', "("..self.db.char.WFS[self.MAINHAND]["WF_ALL_CRITS"].."/"..(self.db.char.WFS[self.MAINHAND]["WF_ALL_PROCS"]*3)..") "..self.WF_ALL_OUT_CRITS.."%"
	)
	cat:AddLine(
		'text', L["Average"]..":",
		'text2', self.WF_ALL_AVERAGE
	)
	cat:AddLine(
		'text', L["Total WF Damage"]..":",
		'text2', self.db.char.WFS[self.MAINHAND]["WF_ALL_TOTAL"]
	)

	if (self.db.profile.tooltipShowHint) then
		T:SetHint(L["Shift-Click to insert your stats into a chat message."])
	end
end

function WindFuryFu:InventoryChanged()
	if (arg1 == "player") then
		self:WeaponCheck()
	end
end

function WindFuryFu:SpellSelfDamage()
	if (self.STATUS) then
		-- Trigger for Windfury Attack
		if (string.find(arg1, L["Windfury"])) then
			
			if (self.DUAL_HIT) then
				self.SWINGS_LEFT = self.SWINGS_LEFT + 2
			elseif (self.SWINGS_LEFT == 0) then
				self.SWINGS_LEFT = 2
				self.THIS_TOTAL = 0
				self.SHIT = 0
				self:AddValue("HITS", 1, "SESSION")
				self:AddValue("WF_ALL_HITS", 1, "LIFETIME")
				self:AddValue("PROCS", 1, "SESSION")
				self:AddValue("WF_ALL_PROCS", 1, "LIFETIME")
			end
			
			local _,creatureName,tdamage
			
			-- Normal Windfury Attack (SPELLLOGSELFOTHER)
			_,creatureName,tdamage = D(arg1, SPELLLOGSELFOTHER)
			if (creatureName ~= nil) then
				self.THIS_TOTAL = self.THIS_TOTAL + tdamage
				self.SHIT = tonumber(tdamage)
				self:AddValue("WF_ALL_TOTAL", tdamage, "LIFETIME")
				self:AddValue("TOTAL", tdamage, "SESSION")		
			else
			
				-- Critical Windfury Attack (SPELLLOGCRITSELFOTHER)
				_,creatureName,tdamage = D(arg1, SPELLLOGCRITSELFOTHER)
				if (creatureName ~= nil) then
					self.THIS_TOTAL = self.THIS_TOTAL + tdamage
					self.SHIT = tonumber(tdamage)
					self.CRIT = self.CRIT + 1
					self:AddValue("WF_ALL_TOTAL", tdamage, "LIFETIME")
					self:AddValue("TOTAL", tdamage, "SESSION")
					self:AddValue("CRITS", 1, "SESSION")
					self:AddValue("WF_ALL_CRITS", 1, "LIFETIME")
				else
				
					-- Dodged Windfury Attack (SPELLDODGEDSELFOTHER)
					_,creatureName = D(arg1, SPELLDODGEDSELFOTHER)
					if (creatureName ~= nil) then
						self:PrintTotal(L["Windfury"] ..": |cffff0000" ..L["DODGED!"])
					else
					
						-- Blocked Windfury Attack (SPELLBLOCKEDSELFOTHER)
						_,creatureName = D(arg1, SPELLBLOCKEDSELFOTHER)
						if (creatureName ~= nil) then
							self:PrintTotal(L["Windfury"] ..": |cffff0000" ..L["BLOCKED!"])
						else
						
							-- Missed Windfury Attack (SPELLMISSSELFOTHER)
							_,creatureName = D(arg1, SPELLMISSSELFOTHER)
							if (creatureName ~= nil) then
								self:PrintTotal(L["Windfury"] ..": |cffff0000" ..L["MISSED!"])
							else
							
								-- Parried Windfury Attack (SPELLPARRIEDSELFOTHER)
								_,creatureName = D(arg1, SPELLPARRIEDSELFOTHER)
								if (creatureName ~= nil) then
									self:PrintTotal(L["Windfury"] ..": |cffff0000" ..L["PARRIED!"])
								else
								
									-- Evaded Windfury Attack (SPELLPARRIEDSELFOTHER)
									_,creatureName = D(arg1, SPELLEVADEDSELFOTHER)
									if (creatureName ~= nil) then
										self:PrintTotal(L["Windfury"] ..": |cffff0000" ..L["EVADED!"])
									else
									
										-- Absorbed Windfury Attack (SPELLLOGABSORBSELFOTHER)
										_,creatureName = D(arg1, SPELLLOGABSORBSELFOTHER)
										if (creatureName ~= nil) then
											self:PrintTotal(L["Windfury"] ..": |cffff0000" ..L["ABSORBED!"])
										else
											self:PrintDebug("Action '" .. arg1 .. "' is unhandled.")
											return
										end
									end
								end
							end
						end
					end
				end
			end

			self:TriggerWindfury()

			if (self.DUAL_HIT) then
				windfuryType = L["Dual Windfury"]
			else
				windfuryType = L["Windfury"]
			end

			if (self.WF_VICTIM.dead == true) then
				self:PrintTotal(windfuryType .." (single death): |cffff0000" .. self.THIS_TOTAL)
				self.WF_VICTIM.dead = false
			elseif (self.SWINGS_LEFT < 1) then
				self:PrintTotal(windfuryType ..": |cffff0000" .. self.THIS_TOTAL)
			end
		-- Trigger for Stormstrike
		elseif (string.find(arg1, BS["Stormstrike"])) then
			local _,creatureName,tdamage
			local isCrit = false
			-- Normal Stormstrike Attack (SPELLLOGSELFOTHER)
			_,creatureName,tdamage = D(arg1, SPELLLOGSELFOTHER)
			if (creatureName ~= nil) then
				self.LASTHIT_CRIT = false			
			else
				-- Critical Stormstrike Attack (SPELLLOGCRITSELFOTHER)
				_,creatureName,tdamage = D(arg1, SPELLLOGCRITSELFOTHER)
				if (creatureName ~= nil) then
					self.LASTHIT_CRIT = true
				end
			end
			
			if (tdamage ~= nil) then
				if ((self.DUAL_WIELD) and (self.SWINGS_LEFT > 0)) then
					self.DUAL_HIT = true
				end
				self.LASTHIT = tdamage
				self:AddValue("HITS", 1, "SESSION")
				self:AddValue("WF_ALL_HITS", 1, "LIFETIME")
				self:Update()
			else
				self.LASTHIT = 0
			end
		end
	end
end

function WindFuryFu:PrintDebug(text)
		if (self.db.profile.debugEnabled) then
			self:Print("Debug: " .. text)
		end
end

function WindFuryFu:PrintTotal(text)
		if (self.db.profile.displayChat) then
			self:Print(text)
		end
		if (self.db.profile.useSCT) then
			local color = self.db.profile.fontColor
			if (IsAddOnLoaded("MikScrollingBattleText")) then
				MikSBT.DisplayMessage(
					text,
					MikSBT.DISPLAYTYPE_OUTGOING, 
					true, 
					self.db.profile.fontColor.r, 
					self.db.profile.fontColor.g, 
					self.db.profile.fontColor.b, 
					self.db.profile.fontSize, 
					self.db.profile.fontFace
				);
			elseif (IsAddOnLoaded("sct")) then
				SCT:DisplayMessage(
					text,
					self.db.profile.fontColor
				);
			end
		end
end

function WindFuryFu:SetVictimInfo()
	local name = UnitName("target")
	if (name ~= nil) then
		local level = UnitLevel("target")
		local _,class = UnitClass("target")
		self.WF_VICTIM.name = name
		self.WF_VICTIM.class = class
		self.WF_VICTIM.level = level
		self.WF_VICTIM.dead = false
	end
end

function WindFuryFu:TargetChanged()
		if (UnitAffectingCombat("player") and (self.STATUS)) then
			self:SetVictimInfo()
		end
end

function WindFuryFu:EnterCombat()
	if (self.STATUS) then
		self:SetVictimInfo()
	end
end

function WindFuryFu:CombatSelfHit()
	if (self.STATUS) then
		local target, amount, isCrit
		isCrit = true
		target, amount = D(arg1, COMBATHITCRITSCHOOLSELFOTHER)
		if not target then
			target, amount, _ = D(arg1, COMBATHITCRITSELFOTHER)
			if not target then
				isCrit = false
				target, amount = D(arg1, COMBATHITSCHOOLSELFOTHER)
				if not target then
					target, amount, _ = D(arg1, COMBATHITSELFOTHER)
					if not target then
						isCrit = true
						target, amount = D(arg1, COMBATHITCRITSCHOOLSELFOTHER, RESIST_TRAILER)
						if not target then
							target, amount, _ = D(arg1, COMBATHITCRITSELFOTHER, RESIST_TRAILER)
							if not target then
								isCrit = false
								target, amount = D(arg1, COMBATHITSCHOOLSELFOTHER, RESIST_TRAILER)
								if not target then
									target, amount, _ = D(arg1, COMBATHITSELFOTHER, RESIST_TRAILER)
								end
							end
						end
					end
				end
			end
		end
		
		if ((self.DUAL_WIELD) and (self.SWINGS_LEFT > 0)) then
			self.DUAL_HIT = true
		end

		if (amount ~= nil) then
			self.LASTHIT = amount
		else
			self.LASTHIT = 0
		end
		
		if (isCrit) then
			self.LASTHIT_CRIT = true
		else
			self.LASTHIT_CRIT = false
		end
		
		self:AddValue("HITS", 1, "SESSION")
		self:AddValue("WF_ALL_HITS", 1, "LIFETIME")
		self:Update()
	end
end

function WindFuryFu:TriggerWindfury()
	local victim = self.WF_VICTIM
	self.SWINGS_LEFT = self.SWINGS_LEFT - 1
	
	if ( self.SHIT > self.db.char.WF[self.MAINHAND]["SHIT"] ) then
		self.db.char.WF[self.MAINHAND]["SHIT"] = self.SHIT
		self.db.char.WF[self.MAINHAND]["SHIT_VICTIM"].name = victim.name
		self.db.char.WF[self.MAINHAND]["SHIT_VICTIM"].class = victim.name
		self.db.char.WF[self.MAINHAND]["SHIT_VICTIM"].level = victim.level
	end
	if ( self.SHIT > self.db.char.WFS[self.MAINHAND]["WF_ALL_SHIT"] ) then
		self.db.char.WFS[self.MAINHAND]["WF_ALL_SHIT"] = self.SHIT
		self.db.char.WFS[self.MAINHAND]["WF_ALL_SHIT_VICTIM"].name = victim.name
		self.db.char.WFS[self.MAINHAND]["WF_ALL_SHIT_VICTIM"].class = victim.name
		self.db.char.WFS[self.MAINHAND]["WF_ALL_SHIT_VICTIM"].level = victim.level
	end
	
	if ((self.SWINGS_LEFT < 1) or (self.WF_VICTIM.dead)) then
		-- If endabled, will add the first hit to the windfury attack total damage
		if (self.db.profile.countFirstHit) then
			self.THIS_TOTAL = self.THIS_TOTAL + self.LASTHIT
			if (self.LASTHIT_CRIT) then
				self:AddValue("CRITS", 1, "SESSION")
			end
		end

		if (self.THIS_TOTAL > self.db.char.WF[self.MAINHAND]["BEST"] ) then
			self.db.char.WF[self.MAINHAND]["BEST"] = self.THIS_TOTAL
			self.db.char.WF[self.MAINHAND]["BEST_VICTIM"].name = victim.name
			self.db.char.WF[self.MAINHAND]["BEST_VICTIM"].class = victim.name
			self.db.char.WF[self.MAINHAND]["BEST_VICTIM"].level = victim.level
		end
		if (self.THIS_TOTAL > self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST"] ) then
			self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST"] = self.THIS_TOTAL
			self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST_VICTIM"].name = victim.name
			self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST_VICTIM"].class = victim.name
			self.db.char.WFS[self.MAINHAND]["WF_ALL_BEST_VICTIM"].level = victim.level
		end
		
		if ((self.DUAL_HIT) and(self.ACTIVE_WEAPON== "both")) then
			self:UpdateStats(self.MAINHAND, true)
			self:UpdateStats(self.OFFHAND, true)
		else
			if (self.ACTIVE_WEAPON== "mainhand") then
				self:UpdateStats(self.MAINHAND, false)
			elseif  (self.ACTIVE_WEAPON== "offhand") then
				self:UpdateStats(self.OFFHAND, false)
			end
		end

		self.LASTHIT = 0
		self.LASTHIT_CRIT = false
		self.CRIT = 0
		self.SWINGS_LEFT = 0
		self.DUAL_HIT = false
	end
	self:Update()
end

function WindFuryFu:UpdateStats(weapon, dualHit)
	self.db.char.WF[weapon]["CRIT"][5] = self.db.char.WF[weapon]["CRIT"][4]
	self.db.char.WF[weapon]["CRIT"][4] = self.db.char.WF[weapon]["CRIT"][3]
	self.db.char.WF[weapon]["CRIT"][3] = self.db.char.WF[weapon]["CRIT"][2]
	self.db.char.WF[weapon]["CRIT"][2] = self.db.char.WF[weapon]["CRIT"][1]

	self.db.char.WF[weapon]["LAST"][5] = self.db.char.WF[weapon]["LAST"][4]
	self.db.char.WF[weapon]["LAST"][4] = self.db.char.WF[weapon]["LAST"][3]
	self.db.char.WF[weapon]["LAST"][3] = self.db.char.WF[weapon]["LAST"][2]
	self.db.char.WF[weapon]["LAST"][2] = self.db.char.WF[weapon]["LAST"][1]
	
	if (dualHit) then
		self.db.char.WF[weapon]["CRIT"][1] = self.CRIT / 2
		self.db.char.WF[weapon]["LAST"][1] = self.THIS_TOTAL / 2
	else
		self.db.char.WF[weapon]["CRIT"][1] = self.CRIT
		self.db.char.WF[weapon]["LAST"][1] = self.THIS_TOTAL
	end

	self.db.char.WF[weapon]["LAST_VICTIM"][5].name = self.db.char.WF[weapon]["LAST_VICTIM"][4].name
	self.db.char.WF[weapon]["LAST_VICTIM"][5].class = self.db.char.WF[weapon]["LAST_VICTIM"][4].class
	self.db.char.WF[weapon]["LAST_VICTIM"][5].level = self.db.char.WF[weapon]["LAST_VICTIM"][4].level
	
	self.db.char.WF[weapon]["LAST_VICTIM"][4].name = self.db.char.WF[weapon]["LAST_VICTIM"][3].name
	self.db.char.WF[weapon]["LAST_VICTIM"][4].class = self.db.char.WF[weapon]["LAST_VICTIM"][3].class
	self.db.char.WF[weapon]["LAST_VICTIM"][4].level = self.db.char.WF[weapon]["LAST_VICTIM"][3].level
	
	self.db.char.WF[weapon]["LAST_VICTIM"][3].name = self.db.char.WF[weapon]["LAST_VICTIM"][2].name
	self.db.char.WF[weapon]["LAST_VICTIM"][3].class = self.db.char.WF[weapon]["LAST_VICTIM"][2].class
	self.db.char.WF[weapon]["LAST_VICTIM"][3].level = self.db.char.WF[weapon]["LAST_VICTIM"][2].level
	
	self.db.char.WF[weapon]["LAST_VICTIM"][2].name = self.db.char.WF[weapon]["LAST_VICTIM"][1].name
	self.db.char.WF[weapon]["LAST_VICTIM"][2].class = self.db.char.WF[weapon]["LAST_VICTIM"][1].class
	self.db.char.WF[weapon]["LAST_VICTIM"][2].level = self.db.char.WF[weapon]["LAST_VICTIM"][1].level
	
	self.db.char.WF[weapon]["LAST_VICTIM"][1].name = self.WF_VICTIM.name
	self.db.char.WF[weapon]["LAST_VICTIM"][1].class = self.WF_VICTIM.class
	self.db.char.WF[weapon]["LAST_VICTIM"][1].level = self.WF_VICTIM.level
end

function WindFuryFu:CombatHostileDeath()
	if (self.STATUS) then
		-- SELFKILLOTHER = You have slain %s!
		local name = D(arg1, SELFKILLOTHER)
		if (name == self.WF_VICTIM.name) then
			self.WF_VICTIM.dead = true
		end
	end
end

function WindFuryFu:ResetStats(type)
	if (type == "SESSION") then
		self:SetupWeapon(self.MAINHAND, "SESSION")
	elseif (type == "LIFETIME") then
		self:SetupWeapon(self.MAINHAND, "LIFETIME")
	end
	self:Update()
end

function WindFuryFu:WeaponCheck()
	self.STATUS = self:WeaponsBuffed()
	self.DUAL_WIELD = false
	local mainHandLink = GetInventoryItemLink("player", GetInventorySlotInfo("MainHandSlot"))
	local offHandLink = nil
	local mainHandName = ""
	local offHandName = ""
	local activeWeapon = ""
	
	if (mainHandLink ~= nil) then
		local _,_,_,_,_,_,weaponType = GetItemInfo(mainHandLink);
		if ((OffhandHasWeapon() == 1) and (string.find(weaponType, "Two-Handed") == nil)) then
			offHandLink = GetInventoryItemLink("player", GetInventorySlotInfo("SecondaryHandSlot"))
		end
	end

	if ((mainHandLink == nil) and (offHandLink == nil)) then
		mainHandName = L["No Weapon Equipped"]
		activeWeapon = "none"
	elseif ((mainHandLink == nil) and (offHandLink ~= nil)) then
		local _, _,itemCode = strfind(offHandLink, "(%d+):")
		offHandName = GetItemInfo(itemCode)
		activeWeapon = "offhand"
	elseif ((mainHandLink ~= nil) and (offHandLink ~= nil)) then
		self.DUAL_WIELD = true
		local _, _, itemCode1 = strfind(mainHandLink, "(%d+):")
		local _, _, itemCode2 = strfind(offHandLink, "(%d+):")
		mainHandName = GetItemInfo(itemCode1)
		offHandName = GetItemInfo(itemCode2)
		activeWeapon = "both"
	else
		local _, _, itemCode = strfind(mainHandLink, "(%d+):")
		mainHandName = GetItemInfo(itemCode)
		activeWeapon = "mainhand"
	end
	
	self.ACTIVE_WEAPON= activeWeapon
	self.MAINHAND = mainHandName
	self.OFFHAND = offHandName
	
	if (self.MAINHAND ~= "") then
		if (self.db.char.WFS[self.MAINHAND] == nil) then
			self:SetupWeapon(self.MAINHAND, "LIFETIME")
		end
		if (self.db.char.WF[self.MAINHAND] == nil) then
			self:SetupWeapon(self.MAINHAND, "SESSION")
		end
		-- Added this check for compatibility with 2.0b1
		if (self.db.char.WFS[self.MAINHAND].WF_ALL_SHIT_VICTIM == nil) then
			self:SetupWeapon(self.MAINHAND, "LIFETIME")
			self:SetupWeapon(self.MAINHAND, "SESSION")
		end
	end
	
	if (self.OFFHAND ~= "") then
		if (self.db.char.WFS[self.OFFHAND] == nil) then
			self:SetupWeapon(self.OFFHAND, "LIFETIME")
		end
		if (self.db.char.WF[self.OFFHAND] == nil) then
			self:SetupWeapon(self.OFFHAND, "SESSION")
		end
		-- Added this check for compatibility with 2.0b1
		if (self.db.char.WFS[self.OFFHAND].WF_ALL_SHIT_VICTIM == nil) then
			self:SetupWeapon(self.OFFHAND, "LIFETIME")
			self:SetupWeapon(self.OFFHAND, "SESSION")
		end
	end
	self:Update()
end

function WindFuryFu:WeaponsBuffed()
	self.MAINHAND_EN = false
	self.OFFHAND_EN = false
	if (GetWeaponEnchantInfo()) then
		local mainHand = G:SetInventoryItem("player", GetInventorySlotInfo("MainHandSlot"))
		if (mainHand) then
			if (G:Find(L["Windfury"]) and not G:Find(BS["Windfury Totem"])) then
				self.MAINHAND_EN = true
			end
		end
		local offHand = G:SetInventoryItem("player", GetInventorySlotInfo("SecondaryHandSlot"))
		if (offHand) then
			if (G:Find(L["Windfury"]) and not G:Find(BS["Windfury Totem"])) then
				self.OFFHAND_EN = true
			end
		end
		
		if ((self.MAINHAND_EN) or (self.OFFHAND_EN)) then
			return true
		end
	end
	return false
end

function WindFuryFu:AddValue(var, val, type)
	if ( type == "LIFETIME" ) then
		self.db.char.WFS[self.MAINHAND][var] = self.db.char.WFS[self.MAINHAND][var] + val
	elseif ( type == "SESSION" ) then
		self.db.char.WF[self.MAINHAND][var] = self.db.char.WF[self.MAINHAND][var] + val
	end
end

function WindFuryFu:SetupWeapon(weapon, type)
	self:PrintDebug("Setting up weapon '" ..weapon.."' for " ..type..".")
	if ( type == "LIFETIME" ) then
		self.db.char.WFS[weapon] = {}
		self.db.char.WFS[weapon].WF_ALL_TOTAL = 0
		self.db.char.WFS[weapon].WF_ALL_PROCS = 0
		self.db.char.WFS[weapon].WF_ALL_HITS = 0
		self.db.char.WFS[weapon].WF_ALL_CRITS = 0
		
		self.db.char.WFS[weapon].WF_ALL_SHIT = 0
		self.db.char.WFS[weapon].WF_ALL_SHIT_VICTIM = {}
		self.db.char.WFS[weapon]["WF_ALL_SHIT_VICTIM"].name = "-"
		self.db.char.WFS[weapon]["WF_ALL_SHIT_VICTIM"].class = ""
		self.db.char.WFS[weapon]["WF_ALL_SHIT_VICTIM"].level = 0
		
		self.db.char.WFS[weapon].WF_ALL_BEST = 0
		self.db.char.WFS[weapon].WF_ALL_BEST_VICTIM = {}
		self.db.char.WFS[weapon]["WF_ALL_BEST_VICTIM"].name = "-"
		self.db.char.WFS[weapon]["WF_ALL_BEST_VICTIM"].class = ""
		self.db.char.WFS[weapon]["WF_ALL_BEST_VICTIM"].level = 0
	elseif ( type == "SESSION" ) then
		self.db.char.WF[weapon] = {}
		
		self.db.char.WF[weapon].SHIT = 0
		self.db.char.WF[weapon].SHIT_VICTIM = {}
		self.db.char.WF[weapon]["SHIT_VICTIM"].name = "-"
		self.db.char.WF[weapon]["SHIT_VICTIM"].class = ""
		self.db.char.WF[weapon]["SHIT_VICTIM"].level = 0
		
		self.db.char.WF[weapon].BEST = 0
		self.db.char.WF[weapon].BEST_VICTIM = {}
		self.db.char.WF[weapon]["BEST_VICTIM"].name = "-"
		self.db.char.WF[weapon]["BEST_VICTIM"].class = ""
		self.db.char.WF[weapon]["BEST_VICTIM"].level = 0
		
		self.db.char.WF[weapon].LAST = { 0,0,0,0,0 }
		self.db.char.WF[weapon].LAST_VICTIM = {1,2,3,4,5}
		
		self.db.char.WF[weapon]["LAST_VICTIM"][1] = {}
		self.db.char.WF[weapon]["LAST_VICTIM"][1].name = "-"
		self.db.char.WF[weapon]["LAST_VICTIM"][1].class = ""
		self.db.char.WF[weapon]["LAST_VICTIM"][1].level = 0
		
		self.db.char.WF[weapon]["LAST_VICTIM"][2] = {}
		self.db.char.WF[weapon]["LAST_VICTIM"][2].name = "-"
		self.db.char.WF[weapon]["LAST_VICTIM"][2].class = ""
		self.db.char.WF[weapon]["LAST_VICTIM"][2].level = 0
		
		self.db.char.WF[weapon]["LAST_VICTIM"][3] = {}
		self.db.char.WF[weapon]["LAST_VICTIM"][3].name = "-"
		self.db.char.WF[weapon]["LAST_VICTIM"][3].class = ""
		self.db.char.WF[weapon]["LAST_VICTIM"][3].level = 0
		
		self.db.char.WF[weapon]["LAST_VICTIM"][4] = {}
		self.db.char.WF[weapon]["LAST_VICTIM"][4].name = "-"
		self.db.char.WF[weapon]["LAST_VICTIM"][4].class = ""
		self.db.char.WF[weapon]["LAST_VICTIM"][4].level = 0
		
		self.db.char.WF[weapon]["LAST_VICTIM"][5] = {}
		self.db.char.WF[weapon]["LAST_VICTIM"][5].name = "-"
		self.db.char.WF[weapon]["LAST_VICTIM"][5].class = ""
		self.db.char.WF[weapon]["LAST_VICTIM"][5].level = 0
		
		self.db.char.WF[weapon].CRIT = { 0,0,0,0,0 }
		self.db.char.WF[weapon].HITS = 0
		self.db.char.WF[weapon].CRITS = 0
		self.db.char.WF[weapon].PROCS = 0
		self.db.char.WF[weapon].TOTAL = 0
	end
end